/*
 * @copyright		© Copyright 2006-2011, epages GmbH, All Rights Reserved.
 *
 * @module			de_epages.order.ui.documentGenerator
 *
 * @revision		$Revision: 1.1.2.1.2.2 $
 */

jQuery.plugin( 'de_epages.order.ui.documentGenerator', {
	plugin:	['jQuery.tmpl', 'jQuery.metaparse', 'jQuery.ui.widget', 'jQuery.ui.draggable', 'ep.core','de_epages.core', 'ep.dict','ep.ui.input', 'ep.ui.dialog', 'ep.ajax', 'ep.ui.datepicker', 'jQuery.i18n'],
	tmpl:	'de_epages.order.ui.documentgenerator',
	dict:	['de_epages.order.dictionary', 'ep.dict']
}, function($){

/*
 * @dictionary		de_epages.order.dictionary
 *
 * @translation		{SetStatusInvoiced}
 *					{DeliveryDate}
 *					{SetStatusDispatched}
 *					{SelectedOrders}
 *					{Invoice}
 *					{SetStatusReadyForShipping}
 *					{PackingSlip}
 *					{CreateDocuments}
 *					{Documents}
 *					{InvoicePackingSlipDate}
 *					{CreateAndPrintDocuments}
 *					{WaitForGenerateDocuments}
 */

/*
 * @dictionary		ep.dict
 *
 * @translation		{Create}
 *					{Cancel}
 *					{Action}
 */
 var dict = $.dict($.dictionary(['ep.dict','de_epages.order.dictionary']));

	$.widget( 'de_epages.order.ui.DocumentGenerator', {

		dict: dict,
		options: {
			orders    :   undefined,
			callback  :   undefined
		},

		_create: function(){
			var self = this;
			$.extend(
				self,
				$.tmpl('de_epages.order.ui.documentgenerator')
					.dictParse(dict,true)
					.tmplItem('elements')
			);
			self.dialog
				.find('.ep-js')
				.metaparse();
			if(this.options.orders !== undefined){
				self.dialog
					.find('h4:first')
					.text( (self.dict.translate('{SelectedOrders}') + ": " + this.options.orders.length) );
			}
			ep(self.dialog).uiDialog({
				title: '<span>'+self.dict.translate('{CreateAndPrintDocuments}')+'</span>',
				modal: true,
				width: '400',
				height: '440',
				buttons: {
					'Generate' : {
						text: self.dict.translate('{Create}'),
						id: 'Generate',
						click: function() {
							self._generate(this);
						}
					},
					'Close': {
						text: self.dict.translate('{Cancel}'),
						click: function() {
							ep(this).uiDialog('close');
						}
					}
				},
				autoOpen: false,
				create: function() {
					self._validate()
				}
			});
		},

		_updateOrder: function(readyForShipping,dispatched,invoiced){
			readyForShipping ? $("#Check_ReadyForShippingOn").attr("checked",true) : $("#Check_ReadyForShippingOn").attr("checked",false);
			dispatched ? $("#Check_DispatchedOn").attr("checked",true) : $("#Check_DispatchedOn").attr("checked",false);
			invoiced ? $("#Check_InvoicedOn").attr("checked",true) : $("#Check_InvoicedOn").attr("checked",false);
		},

		_generate: function(widget){
			var self = this;
			if (!$("#Generate").hasClass("Disabled")){
				var create = this.create.attr("checked")?true:false,
					createandprint = this.createAndPrint.attr("checked")?true:false,
					createPackingSlip = this.CreatePackingSlip.attr("checked")?true:false,
					createInvoice = this.CreateInvoice.attr("checked")?true:false,
					packingSlipDate = this.PackingSlipDate.val(),
					deliveryDate = this.DeliveryDate.val(),
					readyForShipping = this.ReadyForShipping.attr("checked")?true:false,
					dispatched = this.Dispatched.attr("checked")?true:false,
					invoiced = this.Invoiced.attr("checked")?true:false,
					objectId = ep.config.objectId;

				ep.ajax({
					dataType: "json",
					// use a synchronous request to avoid pop up blockers when using window.open
					async: false,
					data: {
						'ObjectID' : ep.config.siteId,
						'OrderList': (self.options.orders) ? self.options.orders : [objectId],
						'ChangeAction' : 'JSONGenerateDocuments',
						'Locale' : epages.vars.Locale.alias,
						'DeliveryDate' : deliveryDate,
						'IssueDate' : packingSlipDate,
						'createInvoice' : createInvoice,
						'createPackingSlip' : createPackingSlip,
						'ReadyForShippingOn' : readyForShipping,
						'DispatchedOn' : dispatched,
						'InvoicedOn' : invoiced
					},
					cache: false,
					type: "post",
					traditional: true
				})
				.done(function(data){
					if(typeof self.options.callback == "function"){
					self.options.callback(data);
					}

					self._updateOrder(readyForShipping,dispatched,invoiced);
					self._printView();
					ep(widget).uiDialog('close');
					self.dialog.remove();
				});
			}
		},

		_printView: function(){
			var printString,
				printPackingSlips = $("#Documents_PackingSlip").attr("checked") ? 1 : 0,
				printInvoices = $("#Documents_Invoice").attr("checked") ? 1 : 0,
				partOfString = "&PopUp=1&Print=1&PrintPackingSlips="+printPackingSlips+"&PrintInvoices="+printInvoices;
			if(this.options.orders !== undefined && ( this.options.orders.length > 1 || ep.config.siteId == ep.config.objectId ) ){
				var list = this.options.orders,
					printString="?ViewAction=MBO-ViewPrintMultipleDocuments&ObjectID="+ep.config.objectId+partOfString;
				$.each(list,function(index, value){
					printString = printString + '&'+'ListObjectID=' + value;
				});
			}
			else{
				printString="?ViewAction=MBO-ViewPrintMultipleDocuments&ObjectID="+ep.config.siteId+partOfString+"&ListObjectID="+ep.config.objectId;
			}
			$("#DocumentGeneratorMessage")
				.removeClass("HideElement")
				.find("a")
				.attr("href", printString);
			if (this.createAndPrint.attr("checked")){
				window.open(printString);
			}
		},

		_validate: function(){
			 var self = this;
			 $.each([self.CreatePackingSlip, self.CreateInvoice], function(index, value){
				this.on('change', function(){
					$("#Generate")[(!self.CreatePackingSlip.is(":checked") && !self.CreateInvoice.is(":checked"))? 'addClass' : 'removeClass']("Disabled");
				});
			});
			$.each([self.PackingSlipDate, self.DeliveryDate], function(index, value){
				this.on('change', function(){
					$("#Generate")[(self.PackingSlipDate.val()=="" || self.DeliveryDate.val()=="")? 'addClass' : 'removeClass']("Disabled");
				});

			});
		},

		open: function(){
			var self = this;
			if(self.dialog === undefined){
			self._create();
			}
			else{
				ep(self.dialog).uiDialog('open');
			}

		},

		_init: function(){
			var self = this,
				epDate = new ep.Date();
			$.each([self.PackingSlipDate, self.DeliveryDate], function(index, value){
				this.val(epDate.getFormat('dd/MM/yyyy'));
			});
		},
		destroy: function(){
			$(".ui-dialog").dialog("destroy").remove();
			this.Inherited(arguments);
			this.dialog.remove();
		}
	});

});