/*
 * @copyright   © Copyright 2006-2012, epages GmbH, All Rights Reserved.
 *
 * @module      de_epages.mediagallery.ui.filemanager
 *
 * @revision    $Revision: 1.1.2.2.2.3 $
 */

jQuery.plugin( "de_epages.mediagallery.filemanager", {
	plugin: [
		"backbone",
		"jQuery.tmpl",
		"jQuery.lazyload",
		"jQuery.ui.widget",
		"jQuery.ui.selectable",
		"jQuery.jstree",
		"jQuery.jstree.themes",
		"jQuery.jstree.json_data",
		"ep.dict",
		"ep.ui.tooltip",
		"de_epages.core",
		"de_epages.mediagallery.ui.memoryusage",
		"de_epages.presentation.ui.uploader"
	],
	dict: [
		"ep.dict",
		"de_epages.mediagallery.dictionary"
	],
	tmpl: [
		"de_epages.mediagallery.fileview",
		"de_epages.mediagallery.folderview",
		"de_epages.mediagallery.toolbarview",
		"de_epages.mediagallery.listview",
		"de_epages.mediagallery.treeview",
		"de_epages.mediagallery.navigationview",
		"de_epages.mediagallery.preview"
	]
}, function( $, undefined ){

	/*
	* @dictionary			ep.dict
	*
	* @translation		{MaxFileSize}
	*									{Open}
	*									{Delete}
	*									{Download}
	*
	* @dictionary			de_epages.mediagallery.dictionary
	*
	* @translation		{NewFolder}
	*									{MediaGallery}
	*									{ImageDb}
	*									{DirectoryAlreadyExists}
	*									{FileAlreadyExists}
	*									{UnsupportedFileFormat}
	*									{TitleDeleteFiles}
	*									{HeadLineDeleteFiles}
	*									{Files}
  *									{BackgroundImages}
	*									{HeaderImages}
	*									{ExtensionDeleteFiles}
	*/

	var epDict = $.dict("ep.dict"),
		mediagalleryDict = $.dict("de_epages.mediagallery.dictionary"),

		// translations
		tToolbar = {
			tNewFolder :            mediagalleryDict.translate("NewFolder"),
			tMaxFileSize :          epDict.translate("MaxFileSize")
		},
		tMediaGallery =           mediagalleryDict.translate("MediaGallery"),
		tBackgroundImages =       mediagalleryDict.translate("BackgroundImages"),
		tHeaderImages =           mediagalleryDict.translate("HeaderImages"),
		tImageDb =                mediagalleryDict.translate("ImageDb"),
		tOpen =                   epDict.translate("Open"),
		tReverseSortOrder =       mediagalleryDict.translate("ReverseSortOrder"),
		tNewFolderName =          mediagalleryDict.translate("NewFolder"),
		tDirectoryAlreadyExists = mediagalleryDict.translate("DirectoryAlreadyExists"),
		tFileAlreadyExists =      mediagalleryDict.translate("FileAlreadyExists"),
		tUnsupportedFileFormat =  mediagalleryDict.translate("UnsupportedFileFormat"),
		tTitleDeleteFiles =       mediagalleryDict.translate("DeleteFiles"),
		tHeadLineDeleteFiles =    mediagalleryDict.translate("HeadLineDeleteFiles"),
		tFiles =                  mediagalleryDict.translate("Files"),
		tExtensionDeleteFiles =   mediagalleryDict.translate("ExtensionDeleteFiles"),
		tPreview =                epDict.translate("Preview"),
		tDelete =                 epDict.translate("Delete"),
		tDownload =               mediagalleryDict.translate("DownloadFile"),

		// defaults
		defaults = {
			//syncLogo
			element:       $("#fileManager"),                       // element which contains the file manager
			accept:        "*/*",                                   // a string of MIME types and/or MIME type groups which will be accepted by the uploader
			selectable:    "image/",                                // a string of MIME types and/or MIME type groups which can be selected
			currentFolder: "MediaGallery"                           // root node of the opened folder
		};

	// Sync
	//--------------------------------------------------------------------------------------

	// sync method to add, rename amd remove files and folders
	var filemanagerSync = (function(Backbone){
		var sync = function(method, model, options){

					createDirectory = function(){
						ep.ajax({
							dataType: "json",
							type : "POST",
							data : {
								ChangeAction : "JSONCreateDirectory",
								DirectoryName : model.get("data"),
								ObjectID : ep.config.siteId,
								TargetDirectory : options.directory,
								ViewAction : "JSONViewResponse"
							}
						})
						.done(function( jsonData, statusText, jqXHR ){
							var data = jsonData[0].children[0];

							data.id = data.metadata.type + data.metadata.path;

							model
								.set(data)
								.trigger("editName");

							options.view
								.trigger("updateUsage", jsonData[0].currentGallerySize)
								.trigger("addTreeNode", {
									data: jsonData[0].children[0],
									id: options.view.model.get("id"),
									path: options.view.model.get("metadata").path
								});
						})
						.fail(function( jqXHR, statusText, errorThrown ){
							if(errorThrown[0].Reason === "DirectoryAlreadyExists"){
								model.trigger("directoryAlreadyExists");
							}
						});
					};

					renameDirectory = function(){
						ep.ajax({
							dataType: 'json',
							type : 'POST',
							data : {
								ChangeAction : 'JSONRenameDirectory',
								SourceDirectory : model.get("metadata").path,
								ObjectID : ep.config.siteId,
								TargetDirectoryName : model.get("data"),
								ViewAction : 'JSONViewResponse'
							}
						})
						.done(function( jsonData, statusText, jqXHR ){
							var prevID = model.get("id");

							model.set({
								data: jsonData.ResultingDirectory,
								id: "MediaGallery"+jsonData.ResultingPath,
								metadata: {
									path: jsonData.ResultingPath,
									type: "MediaGallery"
								}
							});
							options.view.nameSaved = false;
							options.view.trigger("renameTreeNode", {model: model, prevID: prevID});
						})
						.fail(function( jqXHR, statusText, errorThrown ){
							options.view.nameSaved = false;
							if(errorThrown[0].Reason === "DirectoryAlreadyExists"){
								model.trigger("directoryAlreadyExists");
							}
						});
					};

					renameFile = function(){
						ep.ajax({
							dataType: 'json',
							type : 'POST',
							data : {
								ChangeAction : 'JSONRenameFile',
								SourceFile : options.prevName,
								ObjectID : ep.config.siteId,
								TargetFileName : model.get("name"),
								ViewAction : 'JSONViewResponse'
							}
						})
						.done(function( jsonData, statusText, jqXHR ){
							model.set({
								name: jsonData.ResultingFilename,
								placeholder: jsonData.ResultingFullPath,
								path: jsonData.ResultingPath,
								fullpath: jsonData.ResultingFullPath
							});
							options.view.nameSaved = false;
						})
						.fail(function( jqXHR, statusText, errorThrown ){
							options.view.nameSaved = false;
							if(errorThrown[0].Reason === "FileAlreadyExists"){
								model.trigger("fileAlreadyExists");
							}
						});
					};

					deleteFile = function(){
						ep.ajax({
							dataType: 'json',
							type : 'POST',
							data : {
								ChangeAction : 'JSONDeleteFile',
								File : model.get("path"),
								ObjectID : ep.config.siteId,
								ViewAction : 'JSONViewResponse'
							}
						})
						.done(function( jsonData, statusText, jqXHR ){
							options.view
								.trigger("updateUsage", jsonData.currentGallerySize)
								.trigger("removeExistingFile", model.get("name"));
						});
					};

					deleteDirectory = function(){
						ep.ajax({
							dataType: 'json',
							type : 'POST',
							data : {
								ChangeAction : 'JSONDeleteDirectory',
								Directory : model.get("metadata").path,
								ObjectID : ep.config.siteId,
								ViewAction : 'JSONViewResponse'
							}
						})
						.done(function( jsonData, statusText, jqXHR ){
							options.view
								.trigger("updateUsage", jsonData.currentGallerySize)
								.trigger("removeTreeNode", {id: model.get("id")});
						});
					};

					switch(options.method){
						case "renameFile":
							renameFile();
							break;
						case "renameDirectory":
							renameDirectory();
							break;
						case "createDirectory":
							createDirectory();
							break;
						case "deleteFile":
								deleteFile();
								break;
						case "deleteDirectory":
							deleteDirectory();
							break;
					}
				};

		return sync;
	})(Backbone);


	// Models
	//--------------------------------------------------------------------------------------

	// module: ItemModel
	var ItemModel = (function(Backbone){
		var model = Backbone.Model.extend({
			defaults: {}
		});

		return model;
	})(Backbone);

	// module: FileModel
	var FileModel = (function(Backbone){
		var model = Backbone.Model.extend({
			sync: filemanagerSync,
			defaults: {
				modelType: "file",
				placeholder: ep.config.javascriptRoot+"/de_epages/mediagallery/images/placeholder.png"
			},

			initialize: function(){
				var displayableExtensions = [".jpg", ".jpeg", ".png", ".gif"]
						extension = this.get("extension");

				// check whether a file is displayable or not
				if( jQuery.inArray( extension.toLowerCase(), displayableExtensions) === -1 ){
					this.set({
						preview: ep.config.javascriptRoot+"/de_epages/mediagallery/images/ico_unknowndocument.png",
						largePreview: ep.config.javascriptRoot+"/de_epages/mediagallery/images/ico_unknowndocumentpreview.png",
						icon: true
					});
				}
			}
		});

		return model;
	})(Backbone);

	// module: FolderModel
	var FolderModel = (function(Backbone){
		var model = Backbone.Model.extend({
			sync: filemanagerSync,
			defaults: {
				icon: ep.config.javascriptRoot+"/de_epages/mediagallery/images/ico_folder.png",
				modelType: "folder"
			},

			initialize: function(){
				// set id for foldermodels
				if(this.get("metadata")){
					this.set({id: this.get("metadata").type + this.get("metadata").path});
				}
			}
		});

		return model;
	})(Backbone);



	// Collections
	//--------------------------------------------------------------------------------------

	// module: ItemCollection
	var ItemCollection = (function(Backbone, ItemModel){
		var collection =  Backbone.Collection.extend({
			model: ItemModel
		});

		return collection;
	})(Backbone, ItemModel);

	// module: FileCollection
	var FileCollection = (function(Backbone, ItemCollection, FileModel){
		var collection =  ItemCollection.extend({
			model: FileModel
		});

		return collection;
	})(Backbone, ItemCollection, FileModel);

	// module: FolderCollection
	var FolderCollection = (function(Backbone, ItemCollection, FolderModel){
		var collection = ItemCollection.extend({
			model: FolderModel
		});

		return collection;
	})(Backbone, ItemCollection, FolderModel);



	// Views
	//--------------------------------------------------------------------------------------

	// module: ToolbarView
	// create new folders, add files and show memory usage
	var ToolbarView = (function($, Backbone){
		var view = Backbone.View.extend({
			tagName:   "div",
			template:  "de_epages.mediagallery.toolbarview",

			events: {
				"click .ep-mediagallery-newFolderButton" : "triggerNewFolder"
			},

			initialize: function(){
				this.currentUsage = this.options.currentUsage;
				this.currentDirectory = "";
				this.existingFiles = this.getExistingFiles(this.options.existingFiles);

				// set general data for file upload
				this.uploaderData = {
					ChangeAction: "UploadFilesToMediaGallery",
					Directory: this.currentDirectory,
					ErrorAction: "JSONAjaxTransportFileError",
					ObjectID: ep.config.siteId,
					ViewAction: "JSONViewResponse",
					handleashtml:	1
				};

				this
					// dis- or enable the toolbar buttons according to the opened folder
					// you can only edit items in the mediagallery folder
					.on("folderEditable", this.changeButtonState, this)
					// update the memory usage when something changes
					.on("updateUsage", this.updateUsage, this);
			},

			render: function(){
				var self = this,
					options = $.extend(tToolbar, {maxFileSize: Math.floor(ep.config.maxContentLength/1000)}),
					tmpl = $.tmpl(self.template, options);

				self.setElement(tmpl);

				$.extend(self, $.tmplItem(tmpl).elements);

				// initialize uploader
				// interaction with the server - the upload handling should be moved to the sync method someday
				self.uploader = de_epages(self.uploaderInput)
					.presentationUiUploader({
						data:       this.uploaderData,
						multiple:   true,
						exists:     self.existingFiles,
						accept:     this.options.accept,
						callback:   function( data, statustext ){
							if( statustext === "success"){
								var folderData = {
									type:   "MediaGallery",
									path:   self.currentDirectory,
									reload: true
								};

								self.updateUsage(data[0].success.currentGallerySize);
								self.trigger("openFolder", folderData);
							}
						}
					});

				// initialize memory usage
				self.memoryUsage = de_epages(self.memoryUsageContainer)
					.mediagalleryUiMemoryusage({
						maxSize:      self.options.maxSize,
						currentUsage: self.currentUsage
					});

				if( self.options.currentDirectory !== "MediaGallery" ){
					self.changeButtonState(false);
				}

				return self;
			},

			// get an array of the names of the existing files in the current directory
			// this is needed for the uploader to check whether a file already exists
			getExistingFiles: function(collection){
				var filesArray = [];

				if( _.isArray(collection) ){
					_.each(collection, function(item){
						filesArray.push(item.name);
					});
				}
				else{
					collection.each( function(item){
						filesArray.push(item.get("name"));
					});
				}

				return filesArray;
			},

			// remove file from existing files array
			removeExistingFile: function(name){
				for(var i=0; i < this.existingFiles.length; i++){
					if(this.existingFiles[i] === name){
						this.existingFiles.splice(i, 1);
					}
				}
				this.updateUploaderData();
			},

			// update uploader data and existing files when current folder changes
			updateCurrentDirectory: function(data){
				this.uploaderData.Directory = this.currentDirectory = data.path;
				this.existingFiles = this.getExistingFiles(data.files);
				this.updateUploaderData();
			},

			// update uploader options
			updateUploaderData: function(){
				this.uploader.presentationUiUploader("option", {
					data: this.uploaderData,
					exists: this.existingFiles
				});
			},

			// disable toolbar buttons if an folder isn't editable
			changeButtonState: function(editable){
				var uploadButton = this.$(".ep-mediagallery-uploadButton");

				if(editable){
					this.newFolderButton
						.removeAttr("disabled")
						.removeClass("disabled");
					this.uploaderInput
						.removeAttr("disabled");
					uploadButton
						.off("click");
				}
				else{
					this.newFolderButton
						.attr("disabled", "disabled")
						.addClass("disabled");
					this.uploaderInput
						.attr("disabled", "disabled");
					uploadButton
						.on("click", function(event){
							event.preventDefault();
						});
				}
			},

			// update memory usage
			updateUsage: function(value){
				this.currentUsage = value;
				this.memoryUsage.mediagalleryUiMemoryusage("updateUsage", this.currentUsage);
			},

			// trigger event if new folder was created
			triggerNewFolder: function(){
				this.trigger("createNewFolder");
			}
		});

		return view;
	})(jQuery, Backbone);

	// module: TreeView
	// displays a single jstree in the navigation view for every main navigation element
	var TreeView = (function($, Backbone){
		var view = Backbone.View.extend({
			tagName:   "li",
			template:  "de_epages.mediagallery.treeview",

			events: {
				"click .ep-mediagallery-treeHead": "onHeaderClick",
				"click .ep-mediagallery-tree a": "onLinkClick"
			},

			initialize: function(){
				var name = this.model.get("data"),
					editable = (name === "MediaGallery") ? true : false,
					state = (this.options.currentFolder === name) ? "open" : "closed";

				this.model.set({
					state: state,
					editable: editable
				});

				this.model
					// show or hide a single tree
					// possible states: open/closed
					.on("change:state", this.setState, this)
					// add a new node to the tree
					.on("addTreeNode", this.addTreeNode, this)
					// change the name of a tree node
					.on("renameTreeNode", this.renameTreeNode, this)
					// remove a tree node when a folder was deleted
					.on("removeTreeNode", this.removeTreeNode, this)
					// mark the currently selected node
					.on("setActiveNode", this.setActiveNode, this);
			},

			render: function(){
				var self = this,
					name = this.model.get("data"),
					tName = undefined,
					tmpl;

				// translate the given name
				switch(name){
					case "MediaGallery":
						tName = tMediaGallery;
						break;
					case "HeaderImages":
						tName = tHeaderImages;
						break;
					case "BackgroundImages":
						tName = tBackgroundImages;
						break;
					case "ImageDB":
						tName = tImageDb;
						break;
					default:
						break;
				}

				tmpl = $.tmpl(this.template, { name: tName ? tName : name });
				this.setElement(tmpl);

				$.extend(this, $.tmplItem(tmpl).elements);

				_.each(self.options.treeData.children, function(item){
						if(item.metadata.displayname){
							item.data = item.metadata.displayname;
						}

						item.attr = {
							id : item.metadata.type+item.metadata.path,
							title: item.data
						};
				});

				// create jstree
				this.tree.jstree({
					"json_data" : {
						data : self.options.treeData.children,
						ajax : {
							url :      '?',
							type:      'POST',
							dataType:  'json',
							beforeSend: function( xhr, s ){
								xhr.setRequestHeader('Accept', 'text/x-json');
							},
							context: this,
							success: function(data){
								_.each(data[0].children, function(item){
									if(item.metadata.displayname){
										item.data = item.metadata.displayname;
									}

									item.attr = {
										id : item.metadata.type+item.metadata.path,
										title: item.data
									};
								});
								return data[0].children;
							},
							data: function(node){
								return {
									ViewAction :              'JSONGetDirectoryList',
									Type:                     node.data('jstree').type,
									ObjectID :                ep.config.siteId,
									Directory :               node.data('jstree').path,
									IgnoreDirectoryNotFound : 1
								};
							}
						}
				},
				themes : {
					theme : 'bo',
					dots  : true,
					icons : true
				},
				"plugins" : [ "themes", "json_data", "crrm" ]
				})
				.on("click", "a", function(event){
					var data = $(this).parent().data("jstree");

					event.preventDefault();
					self.trigger("openFolder", data);
				});

				if(this.model.get("state") === "closed"){
					this.tree.hide();
				}else{
					this.treeHead.addClass("active");
				}

				return this;
			},

			// create a new tree node when a new folder was created
			addTreeNode: function(node){
				var id = (node.id === "MediaGallery") ? -1 : $("li[id='"+ node.id +"']");

				node.data.attr = {
					id: node.data.id,
					title: node.data.data
				};
				this.tree.jstree("create_node", id, "last", node.data, false,true);

				// set jstree data because the create_node method obviously doens't do this
				this.$("li[id='"+ node.data.id +"']").data("jstree", node.data.metadata);
			},

			// change the name of a tree node when a folder name was changed
			renameTreeNode: function(node){
				this.$("li[id='"+ node.prevID +"']")
					.attr({
						id: node.model.get("id"),
						title: node.model.get("data")
					})
					.data("jstree", {
						type: node.model.get("metadata").type,
						path: node.model.get("metadata").path
					})
					.find("a:first")
						.html('<ins class="jstree-icon">&nbsp;</ins>' + node.model.get("data"));
			},

			// remove a tree node when a folder was removed
			removeTreeNode: function(node){
				this.tree.jstree("delete_node", this.$("li[id='"+ node.id +"']"));
			},

			// mark the current folder as active
			setActiveNode: function(id){
				this.tree.find("a.active").removeClass("active");
				$("li[id='"+ id +"']").find("a:first").addClass("active");
			},

			// show or hide tree by clicking the header
			onHeaderClick: function(){
				if(!this.tree.is(":visible")){
					this.trigger("changeState");
					this.model.set("state", "open");
				}

				this.trigger("folderEditable", this.model.get("editable"));
				this.trigger("openFolder", this.model.get("metadata"));
			},

			// handle click on jstree
			onLinkClick: function(){
				this.trigger("folderChanged", this.model);
			},

			// open or close accordion
			setState: function(){
				if(this.model.get("state") === "open"){
					this.tree
						.show(500)
						.find("a.active").removeClass("active");
					this.treeHead.addClass("active");
				}
				else{
					this.tree.hide(500);
					this.treeHead.removeClass("active");
				}
			}
		});

		return view;
	})(jQuery, Backbone);

	// module: NavigationView
	// displays the whole navigation bar containing the different trees
	var NavigationView = (function($, Backbone){
		var view = Backbone.View.extend({
			tagName:   "div",
			className: "ep-mediagallery-navigation",
			template:  "de_epages.mediagallery.navigationview",

			initialize: function(){
				this
					// trigger events to single tree views
					.on("addTreeNode", this.addTreeNode, this)
					.on("renameTreeNode", this.renameTreeNode, this)
					.on("removeTreeNode", this.removeTreeNode, this);

				this.treeViews = [];

				// create tree view for each main folder
				this.collection.each(function (item, i){
						this.treeViews[i] = new TreeView({
								model: item,
								treeData: this.options.rootFolderData[i],
								currentFolder: this.options.currentFolder
						});
						this.treeViews[i]
							.on("openFolder", this.triggerOpenFolder, this)
							.on("changeState", this.changeState, this)
							.on("folderEditable", this.triggerFolderEditable, this);
				}, this);
			},

			render: function(){
				var tmpl = $.tmpl(this.template);

				this.$el.append(tmpl);

				$.extend(this, $.tmplItem(tmpl).elements);

				_.each(this.treeViews, function (item) {
						this.treeList.append(item.render().el);
				}, this);

				return this;
			},

			addTreeNode: function(data){
				var model = this.collection.where({state: "open"})[0];
				model.trigger("addTreeNode", data);
			},

			renameTreeNode: function(data){
				var model = this.collection.where({state: "open"})[0];
				model.trigger("renameTreeNode", data);
			},

			removeTreeNode: function(data){
				var model = this.collection.where({state: "open"})[0];
				model.trigger("removeTreeNode", data);
			},

			setActiveNode: function(id){
				var model = this.collection.where({state: "open"})[0];
				model.trigger("setActiveNode", id);
			},

			triggerOpenFolder: function(data){
				this.trigger("openFolder", data);
			},

			triggerFolderEditable: function(editable){
				this.trigger("folderEditable", editable);
			},

			// change state of an opened tree to closed
			changeState: function(){
				var model = this.collection.where({state: "open"});
				model[0].set("state", "closed");
			}
		});

		return view;
	})(jQuery, Backbone);

	// module:
	// displays a single folder item
	var FolderView = (function($, Backbone){
		var view = Backbone.View.extend({
			tagName:   "li",
			className: "ep-mediagallery-folderContainer",
			template:  "de_epages.mediagallery.folderview",

			events: {
				"dblclick" : "openFolder",
				"click .ep-mediagallery-itemName" : "edit",
				"keypress input" : "save",
				"blur input" : "save"
			},

			initialize: function(){
				this.model
					// render view when an attribute changes
					.on("change", this.render, this)
					// display a tooltip when a folder with this name already exists
					.on("directoryAlreadyExists", this.directoryAlreadyExists, this)
					// show an input the edit the name of the folder
					.on("editName", this.edit, this);
			},

			render: function(){
				var tmpl = $.tmpl(this.template, {
							name: this.model.get("data"),
							icon: this.model.get("icon"),
							tOpen: tOpen
						});

				$.extend(this, $.tmplItem(tmpl).elements);

				this.$el
					.html(tmpl.children())
					.data("itemID", this.model.get("id"));

				return this;
			},

			edit: function(){
				if(this.model.get("metadata").type === "MediaGallery"){
					this.itemName.replaceWith('<input/>');
					this.input = this.$('input');
					this.input.val(this.model.get("data")).select();
				}
			},

			save: function(e){
				if(	!this.nameSaved && (e.which === 13 || e.which === 27 || e.type === "focusout") ){
					if((e.target.value === this.model.get("data")) || (e.target.value === "")){
						if($(e.target).hasClass("DialogError")){
							return;
						}
						this.render();
					}
					else{
						this.nameSaved = true;
						this.model.save(
							{data: e.target.value},
							{
								wait:   true,
								method: "renameDirectory",
								source: this.model.get("directory"),
								view:   this
							}
						);
					}
				}
			},

			directoryAlreadyExists: function(){
				this.itemName.trigger("click");
				this.input.addClass("DialogError");

				if(this.tooltip){
					this.tooltip.uiTooltip('show');
				}
				else{
					this.tooltip = ep('<span class="Tooltip">'+ tDirectoryAlreadyExists +'</span>')
						.uiTooltip({
							context:     this.input,
							orientation: "right"
						});
				}
			},

			openFolder: function(event){
				if(event.target === this.$('input')[0]){
					return;
				}
				this.trigger("openFolder", this.model.get("metadata"));
			}
		});

		return view;
	})(jQuery, Backbone);


	// module: FileView
	// displays a single file item
	var FileView = (function($, Backbone){
		var view = Backbone.View.extend({
			tagName:   "li",
			className: "ep-mediagallery-fileContainer",
			template:  "de_epages.mediagallery.fileview",

			events: {
				"click .ep-mediagallery-itemName" : "edit",
				"keypress input" : "save",
				"blur input" : "save"
			},

			initialize: function(){
				this.model
					// render view when an attribute changes
					.on("change", this.render, this)
					// display a tooltip when a file with this name already exists
					.on("fileAlreadyExists", this.fileAlreadyExists, this);
			},

			render: function(){
				var tmpl = $.tmpl(this.template, {
					placeholder: this.model.get("placeholder"),
					name:        this.model.get("name"),
					size:        this.model.get("size")/1000 + " kB",
					measure:     (this.model.get("width") !== null) ? this.model.get("width")+" x "+this.model.get("height") : undefined,
					path:        (this.model.get("preview") !== undefined) ? this.model.get("preview") : this.model.get("fullpath")
				});

				$.extend(this, $.tmplItem(tmpl).elements);

				this.$el
					.html(tmpl.children())
					.data("itemID", this.model.get("id"));

				if(!this.model.get("selectable")){
					this.$el.addClass("disabled");
					ep('<span class="Tooltip">'+ tUnsupportedFileFormat +'</span>')
						.uiTooltip({
							context:      this.$el,
							orientation:  "right",
							offsetAdjust: [-60, 30]
						});
				}

				this.$("img.lazy").lazyload({
					container: this.options.list,
					skip_invisible: false
				});

				return this;
			},


			// edit name of file
			edit: function(){
				if(this.model.get("metadata").type === "MediaGallery"){
					var extension = this.model.get("extension"),
						value = this.model.get("name").replace(new RegExp( extension, 'g' ), "") ;

					this.itemName.replaceWith('<input/>');
					this.input = this.$('input');
					this.input
						.val(value)
						.select()
						.after($("<span>").after($("<br/>")).text(extension));
				}
			},

			// save changed name
			save: function(e){
				var extension = this.model.get("extension");

				if( !this.nameSaved && (e.which === 13 || e.which === 27 || e.type === "focusout")  ){
					if((e.target.value === this.model.get("name").replace(new RegExp( extension, 'g' ), "")) || (e.target.value === "")){
						if($(e.target).hasClass("DialogError")){
							return;
						}
						this.render();
					}
					else{
						var prevName = this.model.get("path");
						this.nameSaved = true;

						this.model.save(
							{name: e.target.value + extension},
							{
								wait: true,
								method: "renameFile",
								prevName: prevName,
								view: this
							}
						);
					}

				}
			},

			fileAlreadyExists: function(){
				this.itemName.trigger("click");
				this.input.addClass("DialogError");

				if(this.tooltip){
					this.tooltip.uiTooltip('show');
				}
				else{
					this.tooltip = ep('<span class="Tooltip">'+ tFileAlreadyExists +'</span>')
						.uiTooltip({
							context:     this.input,
							orientation: "right"
						});
				}
			}
		});

		return view;
	})(jQuery, Backbone);

	// module: ListView
	// displays all files and folders from the selected folder
	var ListView = (function($, Backbone, FileCollection, FolderCollection){
		var view = Backbone.View.extend({
			tagName:        "div",
			className:      "ep-mediagallery-listView",
			template:       "de_epages.mediagallery.listview",
			fileCssClass:   "ep-mediagallery-fileContainer",
			folderCssClass: "ep-mediagallery-folderContainer",

			events: {
				"click .changeOrder": "changeOrder"
			},

			initialize: function(){
				this.reverse = false;
			},

			render: function(){
				var self = this,
					name = this.model.get("data"),
					tName,
					tmpl,
					l,
					i;

				switch(name){
					case "MediaGallery":
						tName = tMediaGallery;
						break;
					case "HeaderImages":
						tName = tHeaderImages;
						break;
					case "BackgroundImages":
						tName = tBackgroundImages;
						break;
					case "ImageDB":
						tName = tImageDb;
						break;
					default:
						tName = this.model.get("metadata").displayname ? this.model.get("metadata").displayname : name;
						break;
				}

				tmpl = $.tmpl(this.template, {
					name:              tName,
					reverse:           this.reverse,
					tReverseSortOrder: tReverseSortOrder
				});

				$.extend(this, $.tmplItem(tmpl).elements);

				this.$el.html(tmpl.children());

				// if files isn't already a file collection, save it as that
				if( _.isArray( this.model.get("files") ) ){
					this.model.set({files: new FileCollection(this.model.get("files"))});
				}
				this.files = this.model.get("files");

				// if folders isn't already a folder collection, save it as that
				if( _.isArray( this.model.get("children") ) ){
					this.model.set({children: new FolderCollection(this.model.get("children"))});
				}
				this.folders = this.model.get("children");

				// add file and folder views in the selected order
				if(!this.reverse){
					for(i = 0, l = this.folders.models.length; i < l ; i++){
						this.addFolderView(i);
					}

					for(i = 0, l = this.files.models.length; i < l; i++){
						this.addFileView(i);
					}
				}
				else{
					for(i = this.folders.models.length-1; i >= 0; i--){
						this.addFolderView(i);
					}

					for(i = this.files.models.length-1; i >= 0; i--){
						this.addFileView(i);
					}
				}

				this.activateLazyLoading();

				// set items selectable
				this.list.selectable({
					unselecting: function(event, ui){
						self.$("input").trigger("blur");
					},
					selected: $.proxy(this.selectableselected, this)
				});

				return this;
			},

			addFileView: function(i){
				var item =  this.files.models[i];

				item.set({
					metadata: {type: this.model.get("metadata").type},
					id: item.get("directory") + item.get("path"),
					list: this.$el
				});

				var fileView = new FileView({
						model: item
				});

				this.list.append(fileView.render().el);
			},

			addFolderView: function(i){
				var item =  this.folders.models[i];

				item.set({
						id: item.get("metadata").type + item.get("metadata").path
					});

				var folderView = new FolderView({
						model: item
				});

				folderView
					.on("openFolder", this.triggerOpenFolder, this)
					.on("renameTreeNode", this.renameTreeNode, this);

				this.list.append(folderView.render().el);
			},

			// set lazy loading for all images with class "lazy"
			activateLazyLoading: function(){
				this.$("img.lazy").lazyload({
					container: this.$el,
					skip_invisible: false
				});
			},

			// change display order of file and folder views
			changeOrder: function(){
				this.reverse = this.reverse ? false : true;
				this.render();
			},

			triggerOpenFolder: function(data){
				this.trigger("openFolder", data);
			},

			renameTreeNode: function(data){
				this.trigger("renameTreeNode", data);
			},

			getSelectedElements: function(){
				var models = [];

				this.selected = this.$("li.ui-selected");

				_.each(this.selected, function (item) {
					var $item = $(item),
						itemID = $item.data("itemID");

					if($item.hasClass(this.fileCssClass)){
						models.push(this.files.where({id: itemID})[0]);
					}
					else if($item.hasClass(this.folderCssClass)){
						models.push(this.folders.where({id: itemID})[0]);
					}
				}, this);

				return models;
			},

			// collect selected elements and trigger selected event
			selectableselected: function(){
				this.trigger("selectableselected", this.getSelectedElements());
			},

			// create new folder model/view and save it
			createNewFolder: function(){
				var lastFolder = this.$(".ep-mediagallery-folderContainer:last");
					newFolder = new FolderModel();
					newFolderView = new FolderView({model: newFolder});

				this.folders.add(newFolder);

				newFolder.save(
					{data: tNewFolderName},
					{
						method: "createDirectory",
						directory: this.model.get("metadata").path,
						view: this
					}
				);

				lastFolder.length ? lastFolder.after(newFolderView.render().el) : this.list.prepend(newFolderView.render().el);
				newFolderView
					.on("openFolder", this.triggerOpenFolder, this)
					.on("renameTreeNode", this.renameTreeNode, this);
			},

			// delete the selected items
			deleteItems: function(){
				_.each(this.selected, function (item) {
					var $item = $(item),
							itemID = $item.data("itemID"),
							model;

					if($item.hasClass(this.fileCssClass)){
						model = this.files.where({id: itemID})[0];
						this.files.remove(model);
						model.destroy({method: "deleteFile", view: this});
					}
					if($item.hasClass(this.folderCssClass)){
						model = this.folders.where({id: itemID})[0];
						this.folders.remove(model);
						model.destroy({method: "deleteDirectory", view: this});
					}
				}, this);

				this.selected.remove();
			}
		});

		return view;
	})(jQuery, Backbone, FileCollection, FolderCollection);

	// module: Preview
	// displays the preview of selected files on the right sidebar
	var Preview = (function($, Backbone){
		var view = Backbone.View.extend({
			tagName:   "div",
			className: "ep-mediagallery-preview",
			template:  "de_epages.mediagallery.preview",

			collection: new ItemCollection(),

			events: {
				"click .ep-mediagallery-deleteItem" : "triggerDeleteItems",
				"click .ep-mediagallery-downloadItem" : "downloadItem"
			},

			initialize: function(){
				this.collection
					// when reset is thrown the preview will be cleared
					.on("reset", this.render, this)
					// change preview when something in the collection is changed
					.on("change", this.render, this);
			},

			render: function(){
				var length = this.collection.length,
					model = this.collection.models[0],
					data = {},
					tmpl;

				// translations
				data.tPreview = tPreview;
				data.tDelete = tDelete;
				data.tDownload = tDownload;
				data.name = "";

				if(model){
					// if one item was selected
					if(length === 1){
						if(model.get("modelType") === "file"){
							data.name = model.get("name");
							data.path = (model.get("largePreview") !== undefined) ? model.get("largePreview") : model.get("fullpath");
							data.downloadPath = model.get("fullpath");
							data.downloadable = true;
							data.icon = (model.get("icon") !== undefined) ? true : false;
						}
						else{
							data.name = model.get("data");
							data.path = ep.config.javascriptRoot+"/de_epages/mediagallery/images/ico_preview.png";
							data.downloadable = false;
						}
					}
					// else: serveral items were selected
					else{
						data.path = ep.config.javascriptRoot+"/de_epages/mediagallery/images/ico_multipleselect.png";
					}
					data.type = model.get("metadata").type;
				}

				data.length = length;

				tmpl = $.tmpl(this.template, data);
				this.$el.html(tmpl);

				return this;
			},

			clear: function(){
				this.collection.reset();
				this.render().el;
			},

			triggerDeleteItems: function(){
				this.trigger("deleteItems");
			},

			downloadItem: function(){
				window.open(ep.config.baseUrl+'?ObjectID='+ep.config.siteId+'&ViewAction=View&ChangeAction=ActionDownloadFile&File='+this.collection.models[0].get("path"),'Download');
			}
		});

		return view;
	})(jQuery, Backbone);

	// Main View
	var MainView = Backbone.View.extend({
		tagName:   "div",

		initialize: function(){
			var self = this;

			self.setElement(self.options.element);

			ep.ajax({
				dataType: 'json',
				type : 'POST',
				data : {
					ViewAction :           'JSONInitMediaGallery',
					AllowedFormats:        self.options.selectable,
					ObjectID :             ep.config.siteId,
					ShowMediaGallery :     1,
					ShowHeaderImages :     1,
					ShowBackgroundImages : 1,
					ShowImageDB :          1
				}
			})
			.done(function( jsonData, statusText, jqXHR ){
				self.rootFolderData = jsonData[0].children;
				self.rootFolders = self.prevLoadedFolders = new FolderCollection(self.rootFolderData);
				self.currentFolder = self.rootFolders.where({data: self.options.currentFolder})[0];

				// create default views
				self.list = new ListView({ model: self.currentFolder });
				self.toolbar = new ToolbarView({
					accept:           self.options.accept,
					maxSize:          jsonData[0].maxGallerySize,
					currentUsage:     jsonData[0].currentGallerySize,
					currentDirectory: self.options.currentFolder,
					existingFiles:    self.currentFolder.get("files")
				});
				self.navigation = new NavigationView({
					collection:     self.rootFolders,
					currentFolder:  self.options.currentFolder,
					rootFolderData: self.rootFolderData
				});
				self.preview = new Preview();


				self.render();

				// add event listeners:
				self.list
					// change preview when selection changes
					.on("selectableselected", self.changePreview, self)
					// open a selected folder
					.on("openFolder", self.openFolder, self)
					// trigger events to edit the tree views
					.on("addTreeNode", self.addTreeNode, self)
					.on("renameTreeNode", self.renameTreeNode, self)
					.on("removeTreeNode", self.removeTreeNode, self)
					// remove files from existing files array when they were deleted
					.on("removeExistingFile", self.removeExistingFile, self)
					// update usage after upload or deletion of files
					.on("updateUsage", self.updateUsage, self);
				self.toolbar
					// open a selected folder
					.on("openFolder", self.openFolder, self)
					// create a new folder
					.on("createNewFolder", self.createNewFolder, self);
				self.navigation
					// open a selected folder
					.on("openFolder", self.openFolder, self)
					// trigger folderEditable to change to state of the toolbar buttons
					.on("folderEditable", self.folderEditable, self);
				self.preview
					// delete selected items
					.on("deleteItems", self.deleteItems, self);
			});
		},

		render: function(){
			this.$el
				.addClass("ep-mediagallery")
				.append([
					this.toolbar.render().el,
					this.navigation.render().el,
					this.list.render().el,
					this.preview.render().el
				]);

			this.list.activateLazyLoading();

			// trigger ready event if filemanager is rendered
			this.$el.trigger("filemanagerReady");

			return this;
		},

		// change preview when selection changes
		changePreview: function(models){
			this.preview.collection.reset(models);

			// this is necessary to update the logo in the quick design
			// after 6.16.0 and the new implementation of quick and advanced this can be removed
			if(_.isFunction(this.options.syncLogo)){
				var item = models[0];

				this.options.syncLogo(item.attributes);
			}
		},

		// open a folder in list view
		openFolder: function(data){
			var self = this,
				prevFolder = self.list.model,
				loadedFolder = self.prevLoadedFolders.where({id: data.type + data.path});

			// if the selected folder was previously loaded take the entry from the collection
			if(loadedFolder.length && !data.reload){
				self.updateListView(loadedFolder[0]);
			}
			// otherwise load the unknown folder via ajax
			else{
				ep.ajax({
					dataType: 'json',
					type :    'POST',
					data : {
						ViewAction :              'JSONGetDirectoryList',
						Type :                    data.type,
						AllowedFormats:           self.options.selectable,
						ObjectID :                ep.config.siteId,
						Directory :               data.path,
						IgnoreDirectoryNotFound : 1
					}
				})
				.done(function( jsonData, statusText, jqXHR ){
					if(data.reload){
						loadedFolder[0].set({files: jsonData[0].files});
					}
					else{
						var model = new FolderModel({
							children: jsonData[0].children,
							data:     jsonData[0].data,
							metadata: jsonData[0].metadata,
							files:    jsonData[0].files
						});
					}

					self.updateListView( (model === undefined) ? loadedFolder[0] : model );
				});
			}
		},

		folderEditable: function(editable){
			this.toolbar.trigger("folderEditable", editable);
		},

		// update list view with new model data und reset preview
		updateListView: function(model){
			this.prevLoadedFolders.add(model);
			this.list.model = model;
			this.list.render();
			this.preview.clear();
			this.navigation.setActiveNode(model.get("id"));

			if(model.get("metadata").type === "MediaGallery"){
				this.toolbar.updateCurrentDirectory({
					path:  model.get("metadata").path,
					files: model.get("files")
				});
			}
		},

		// remove deleted file from existing files array
		removeExistingFile: function(name){
			this.toolbar.removeExistingFile(name);
		},

		// trigger updateUsage if files were added/deleted
		updateUsage: function(value){
			this.toolbar.trigger("updateUsage", value);
		},

		addTreeNode: function(data){
			this.navigation.trigger("addTreeNode", data);
		},

		renameTreeNode: function(data){
			this.navigation.trigger("renameTreeNode", data);
		},

		removeTreeNode: function(data){
			this.navigation.trigger("removeTreeNode", data);
		},

		// delete selected items from list view and the corresponding collection
		deleteItems: function(){
			var self = this,
				models = this.getAllSelectedElements(),
				files = "",
				content;

			// create content for ep.alert
			_.each(models, function(item, index){
					files += (item.get("name") ? item.get("name") : item.get("data")) + ( (models.length-1 === index) ? "" : ", ");
			});

			content = '<div class="bold">'+ tHeadLineDeleteFiles +'</div><br/><p>'+ tFiles +': ' + files + '</p><p>'+ tExtensionDeleteFiles + '</p>';

			// create ep.alert
			ep.alert({
				title:   tTitleDeleteFiles,
				content: content,
				type:    "confirmation",
				size:    "xl",
				okTitle: tDelete,
				ok:      function(event){
					self.list.deleteItems();
					self.preview.clear();
				},
				cancel: function(event){}
			});
		},

		// order list view to create a new folder
		createNewFolder: function(){
			this.list.createNewFolder();
		},

		// get the current gallery usage
		getCurrentUsage: function(){
			return this.toolbar.currentUsage;
		},

		// get all currently selected elements from the list view
		getAllSelectedElements: function(){
			return this.list.getSelectedElements();
		},

		// only selectable files will be returned
		getSelectedElements: function(){
			var elements = new ItemCollection(this.list.getSelectedElements()),
					filteredElements = elements.where({selectable: 1});

			return filteredElements;
		}
	});

	$.extend( $.scope('de_epages.mediagallery.filemanager'),{
		init: function(options){
			var _options = $.extend(true, {}, defaults, options),
					filemanager = new MainView(_options);

			return filemanager;
		}
	});

});